// ExperSwarm.m

#import "ExperSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ExperSwarm

+createBegin: (id) aZone {
  ExperSwarm * obj;

  obj = [super createBegin: aZone];

  obj->parameterName      =strdup("numIter");
  obj->lowerBoundParameter=1;
  obj->upperBoundParameter=5;
  obj->incrementParameter =1;
  obj->numGenerations     =100;
  obj->lowerBoundSeed     =8251777;
  obj->upperBoundSeed     =8251778;
   
  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "exper.setup"];
  return [super createEnd];
}

-buildObjects {
  [super buildObjects];
  
  // [probeDisplayManager createProbeDisplayFor: self];
  [controlPanel setStateStopped];
  
  graph = [Graph createBegin: self];
  graph = [graph createEnd];
  [graph setTitle: parameterName];
  [graph setWidth: 400 Height: 300];

  data = [graph createElement];
  [data setLabel: "average payoff"];

  return self;
}



-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  return [self getSwarmActivity];
}


-run {
  double x,y;
  double value;
  double parameter;
  int numRepl;
  int seed;
  int g;
  id <OutFile> outFile;
  id <OutFile> dataFile;
  id <Averager> average;
  char string[100];
  
  value=0.0;
  numRepl=0;

  dataFile=[OutFile create: self withName: parameterName];
  
  for(parameter=lowerBoundParameter;parameter<upperBoundParameter;parameter+=incrementParameter) {
    for(seed=lowerBoundSeed;seed<upperBoundSeed;seed++) {

      // Create setup file for ModelSwarm
      sprintf(string,"@begin\nrandomSeed %d\n%s %f\n@end\n",seed,parameterName,parameter);
      
      outFile=[OutFile create: self withName: "loop.setup"];
      [outFile putString: string];
      [outFile drop];
      
      // Run ModelSwarm
      modelSwarm = [ModelSwarm create: self];
      [ObjectLoader load: modelSwarm fromFileNamed: "loop.setup"];
      [modelSwarm buildObjects];

      // Create an object to calculate statistic
      average=[Averager createBegin: self];
      [average setCollection: [modelSwarm getPlayers]];
      [average setProbedSelector: M(getAveragePayoff)];
      average=[average createEnd];

      // Run the model and get results
      for(g=0;g<numGenerations;g++)
	[modelSwarm step];
      [modelSwarm reportResults];
      value = [[average update] getAverage];

      // Drop the objects used
      [average drop];
      [modelSwarm drop];

      numRepl++;

    }
    [data addX: parameter Y: value];
    [dataFile putDouble: parameter];
    [dataFile putTab];
    [dataFile putDouble: value];
    [dataFile putNewLine];
    
  }
  [graph pack];
  [controlPanel setStateStopped];

  return self;
}




@end








