// ModelSwarm.m				

#import "ModelSwarm.h"
#import <simtools.h>

@implementation ModelSwarm

+createBegin: (id)  aZone {
  ModelSwarm * obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];
  obj->numIter = 4;
  obj->coopPayoff = 3.0;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "randomSeed"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldSize"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "pALLC"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "pTFT"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "pATFT"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "pALLD"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "model.setup"];
  
  return [super createEnd];
}

-buildObjects {
  id aPlayer;
  id index;
  int pt,i,playerID;
  int x,y;
  
  [self setSeed: randomSeed];

  world = [Discrete2d createBegin: self];
  [world setSizeX: worldSize Y: worldSize];
  world = [world createEnd];
  [world fillWithObject: nil];
  
  time = 0;

  numPlayers = worldSize * worldSize;
  num[0] = (int) (numPlayers * pALLC);
  num[1] = (int) (numPlayers * pTFT);
  num[2] = (int) (numPlayers * pATFT);
  num[3] = (int) (numPlayers * pALLD);
 
  playerList = [List create: self];
  
  playerID = 0;
  for (pt = 0; pt < 4; pt++)
    for (i = 0; i < num[pt]; i++) {
      playerID++;
      aPlayer = [Player createBegin: self];
      aPlayer = [aPlayer createEnd];
      [aPlayer initPlayer: playerID type: pt];
      [aPlayer setNeighborhood: [List create: self]];
      [playerList addLast: aPlayer];
    }

  [self shuffle: playerList];
  
  index = [playerList begin: self];
  for (x = 0; x < worldSize; x++) {
    for (y = 0; y < worldSize; y++) {
      aPlayer = [index next];
      [world putObject: aPlayer atX: x Y: y];
      [aPlayer setX: x Y: y];
    }
  }
  [index drop];
  
  [self setNeighborhoods];

  tournament = [Tournament createBegin: self];
  [tournament setNumIter: numIter];
  [tournament setCoopPayoff: coopPayoff];
  tournament = [tournament createEnd];
  
  return self;
}

-setNeighborhoods {
  id index, aPlayer;

  index = [playerList begin: self];
  while ((aPlayer=[index next])) {
    [self setNeighborhood: aPlayer atDX: -1 DY: 0];
    [self setNeighborhood: aPlayer atDX: 1 DY: 0];
    [self setNeighborhood: aPlayer atDX: 0 DY: -1];
    [self setNeighborhood: aPlayer atDX: 0 DY: 1];
  }
  [index drop];
  return self;
}

-setNeighborhood: (id) player atDX: (int) dx DY: (int) dy {
  int x, y;

  x = [player getX];
  y = [player getY];

  if ([self validX: x+dx Y: y+dy]) {
    [[player getNeighborhood] addLast: [world getObjectAtX: x+dx Y: y+dy]];
  }
  return self;
}

-(BOOL) validX: (int) x Y: (int) y {
  return (((x >= 0) && (x < worldSize)) && ((y >= 0) && (y < worldSize)));
}

-(id) getWorld {
  return world;
}

-setSeed: (long int) s {
  randomSeed = s;
  [randomGenerator setStateFromSeed: randomSeed];
  return self;
}

-setSize: (int) s {
  worldSize = s;
  return self;
}

-(int) getSize {
  return worldSize;
}

-setSizeFactor: (int) s {
  sizeFactor = s;
  return self;
}

-(id) getPlayers {
  return playerList;
}

-resetPlayers {
  id index, player;

  index = [playerList begin: self];
  while ((player=[index next])) {
    [player resetPlayer];
  }
  [index drop];
  
  return self;
}

-setNumIter: (int) n {
  numIter = n;
  return self;
}

-setCoopPayoff: (PAYOFF) p {
  coopPayoff = p;
  return self;
}

-(int) getNum0 {
  return num[0];
}

-(int) getNum1 {
  return num[1];
}

-(int) getNum2 {
  return num[2];
}

-(int) getNum3 {
  return num[3];
}

-runTournaments {
  id index, player, neigh, nIndex;
 
  index = [playerList begin: self];
  while ((player=[index next])) {
    nIndex = [[player getNeighborhood] begin: self];
    while ((neigh=[nIndex next]))
      [self runTournament: player against: neigh];
    [nIndex drop];
  }
  [index drop];
  return self;
}

-runTournament: player1 against: player2 {  
  [tournament setPlayer1: player1 Player2: player2];
  [tournament run]; 

  return self;
}

-adaptPlayers {
  id index, player;

  index = [playerList begin: self];
  while ((player = [index next]))
    [player adaptType];
  [index drop];

  index = [playerList begin: self];
  while ((player = [index next]))
    [player updateType];
  [index drop];

  return self;
}

-calcResults {
  id index;
  id aPlayer;
  int pt;
  double avPayoff[4];
  
  for (pt=0; pt<4; pt++) {
    num[pt] = 0;
    avPayoff[pt] = 0.0;
  }

  index = [playerList begin: self];
  while((aPlayer=[index next])) {
    pt = [aPlayer getPlayerType];
    num[pt]++;
    avPayoff[pt] = avPayoff[pt] + [aPlayer getAveragePayoff];
  }
  [index drop];

  return self;
}

-reportResults {

  printf("Seed: %8d Size: %3d Iter: %2d Coop: %5.2f Time: %5d Num: %4d %4d %4d %4d", 
	 randomSeed, worldSize, numIter, coopPayoff, time, num[0], num[1], num[2], num[3]);

//   for (pt=0; pt<4; pt++) {
//     if (num[pt]==0)
//       printf("%6.3f", 0.0);
//     else
//       printf("%6.3f",avPayoff[pt] / num[pt]);
//   }

  printf("\n");
  return self;
}

-step {
  if (time > 0) {
    [self resetPlayers];
    [self runTournaments];
    [self adaptPlayers];
  }
  [self calcResults];
  time++;
  return self;
}

-shuffle: list {
  int j, k;
  id temp;

  j = [list getCount];
  while (j>1) {
    k = [uniformIntRand getIntegerWithMin: 0 withMax: j-1];
    j--;
    temp = [list atOffset: k];
    [list atOffset: k put: [list atOffset: j]];
    [list atOffset: j put: temp];
  } 
  return self;
}

@end



  


