// ObserverSwarm.m

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ObserverSwarm

+createBegin: (id) aZone {
  ObserverSwarm * obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];
  obj->displayFrequency = 4;
  obj->zoomFactor = 1;
  obj->sizeFactor = 20;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "sizeFactor"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

-createEnd {
  return [super createEnd];
}

-buildObjects {
  [super buildObjects];
  modelSwarm = [ModelSwarm create: self];
  [probeDisplayManager createProbeDisplayFor: modelSwarm];
  [probeDisplayManager createProbeDisplayFor: self];

  [controlPanel setStateStopped];
  [modelSwarm buildObjects];
  [modelSwarm setSizeFactor: sizeFactor];
  
  colorMap = [Colormap create: self];
  [colorMap setColor: 0 ToName: "red"];
  [colorMap setColor: 1 ToName: "blue"];
  [colorMap setColor: 2 ToName: "orange"];
  [colorMap setColor: 3 ToName: "darkgrey"];

  worldRaster = [ZoomRaster create: self];
  [worldRaster setColormap: colorMap];
  [worldRaster setZoomFactor: zoomFactor];
  [worldRaster setWidth: sizeFactor*[modelSwarm getSize] Height: sizeFactor*[modelSwarm getSize]];
  [worldRaster setWindowTitle: "Spatial PD"];
  [worldRaster pack];

  worldDisplay = [ModObject2dDisplay createBegin: self];
  [worldDisplay setDisplayWidget: (id <Raster>) self];
  [worldDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [worldDisplay setObjectCollection: [modelSwarm getPlayers]];
  [worldDisplay setDisplayMessage: M(drawSelfOn:)];
  [worldDisplay setSizeFactor: sizeFactor];
  worldDisplay = [worldDisplay createEnd];

  [worldRaster setButton: ButtonRight Client: worldDisplay Message: M(makeProbeAtX:Y:)];  
  numGraph = [EZGraph createBegin: self];
  [numGraph setTitle: "Frequency of strategies"];
  [numGraph setAxisLabelsX: "Time" Y: "Number"];
  numGraph = [numGraph createEnd];
  
  [numGraph createSequence: "all-C" withFeedFrom: modelSwarm andSelector: M(getNum0)]; 
  [numGraph createSequence: "TFT" withFeedFrom: modelSwarm andSelector: M(getNum1)];
  [numGraph createSequence: "aTFT" withFeedFrom: modelSwarm andSelector: M(getNum2)];
  [numGraph createSequence: "all-D" withFeedFrom: modelSwarm andSelector: M(getNum3)];

  [controlPanel setStateStopped];
  return self;
}

-buildActions {
  [super buildActions];

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: modelSwarm         message: M(step)];
  [displayActions createActionTo: worldDisplay       message: M(display)];
  [displayActions createActionTo: worldRaster        message: M(drawSelf)];
  [displayActions createActionTo: numGraph           message: M(step)];
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: 1]; // note frequency!
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
 
  return self;
}

-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  [displaySchedule activateIn: self];
  return [self getSwarmActivity];
}


-drawPointX: (int) posX Y: (int) posY Color: (int) type {
  
  [worldRaster fillRectangleX0: sizeFactor*posX
	       Y0: sizeFactor*posY 
	       X1: sizeFactor*(float)(posX+0.9) 
	       Y1: sizeFactor*(float)(posY+0.9) 
	       Color: type];
	
  return self;
}

@end








