// Player.m				
  
#import "Player.h"
#import <simtools.h>

int iParam[4] = {1, 1, 0, 0};
int pParam[4] = {1, 1, 0, 0};
int qParam[4] = {1, 0, 1, 0};

@implementation Player 

-initPlayer: (int) idnum type: (int) playerType {
   playerID = idnum;
   type = playerType;
   return self;
}

-(int) getID {
  return playerID;
}

-resetPlayer {
  cumulPayoff = 0.0;
  numPlays = 0;
  return self;
}

-setPlayerType: (int) playerType { 
  type = playerType;
  return self;
}


-(int) getPlayerType {
  return type;
}

-setX: (int) x Y: (int) y {
  posX = x;
  posY = y;
  return self;
}

-(int) getX {
  return posX;
}

-(int) getY {
  return posY;
}

-setNeighborhood: (id) nl {
  neighbors = nl;
  cumulPayoff = 0.0;
  numPlays = 0;
  return self;
}

-getNeighborhood {
  return neighbors;
}

-setOtherPlayer: player {
  other = player;
  return self;
}

-setPayoff: (PAYOFF) payoff {
  cumulPayoff = payoff;
  return self;
}

-(PAYOFF) getPayoff {
  return cumulPayoff;
}

-(PAYOFF) getAveragePayoff {
  if (numPlays == 0)
    return 0.0;
  else
    return (PAYOFF) cumulPayoff / (PAYOFF) numPlays;
}

-(int) getNewAction {
  return newAction;
}

-remember {
  memory = [other getNewAction];
  return self;
}

-step: (int) t {
  numPlays++;
  if (t==0)
    newAction = iParam[type];
  else
    if (memory==1)
      newAction = pParam[type];
    else 
      newAction = qParam[type];
  return self;
}

-adaptType {
  id neigh, index; 
  int bestType;
  PAYOFF bestPayoff, currentPayoff;

  bestPayoff = -1.0;
  bestType = type;
  newType = type;

  index = [neighbors begin: [self getZone]];
   while ((neigh = [index next])) {
     currentPayoff = [neigh getAveragePayoff];
     if ((currentPayoff > bestPayoff)) {
       bestPayoff = currentPayoff;
       bestType = [neigh getPlayerType];
     }
     else if (currentPayoff == bestPayoff)
       if ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] < 0.5) {
	 bestPayoff = currentPayoff;
	 bestType = [neigh getPlayerType];
       }
   }
  [index drop];
  
  if (bestPayoff > [self getAveragePayoff])
    newType = bestType;

  return self;
}

-updateType {
  [self setPlayerType: newType];
  return self;
}


-drawSelfOn: (id) raster {
  [raster drawPointX: posX Y: posY Color: type];  
  return self;
}

@end

  


