// ObserverSwarm.m

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ObserverSwarm

+createBegin: (id) aZone {
  ObserverSwarm * obj;

  obj = [super createBegin: aZone];
  obj->displayFrequency = 1;      // default parameter setting
  return obj;
}

-createEnd {
  return [super createEnd];
}

-buildObjects {

  [super buildObjects];

  // First, we create the model that we're actually observing.
  // The model is a subswarm of the observer.

  modelSwarm = [ModelSwarm createBegin: self];
  modelSwarm = [modelSwarm createEnd];

  // Then we ask the model to build itself.

  [modelSwarm buildObjects];

  // Now it's time to build our own display objects.  
  // The numGraph is a graph that will plot the frequency
  // of the four strategies dynamically.

  numGraph = [EZGraph createBegin: self];
  [numGraph setTitle: "Number of all-C"];
  [numGraph setAxisLabelsX: "Time" Y: "Number"];
  numGraph = [numGraph createEnd];

  // Here we make sure that the graph gets dynamic feed from the four strategy types.
  //  [[numGraph getGraph] setScaleModeX: NO Y: YES];
  [numGraph createSequence: "all-C" withFeedFrom: modelSwarm andSelector: M(getNum0)]; 
  [numGraph createSequence: "TFT"   withFeedFrom: modelSwarm andSelector: M(getNum1)];
  [numGraph createSequence: "aTFT"  withFeedFrom: modelSwarm andSelector: M(getNum2)];
  [numGraph createSequence: "all-D" withFeedFrom: modelSwarm andSelector: M(getNum3)];
  
  // Now that we have all display objects, we instruct the control panel to wait for
  // a button event. We halt here until someone hits a control panel button.

  [controlPanel setStateStopped];

  return self;
}

-buildActions
{

  [super buildActions];

  // Create an ActionGroup for both the model and the graph.
  // We refrain from lower-level scheduling inside the modelSwarm, however.

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: modelSwarm         message: M(step)];
  [displayActions createActionTo: numGraph           message: M(step)];
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  // Now we build the actual schedule. Note that the redraw frequency can
  // be manipulated which is useful if we want to save time by now redrawing
  // for every iteration.

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: 1]; 
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
 
  return self;
}

-activateIn: (id) swarmContext {
  // Here we activate the schedules so they are ready to run.
  // The context parameter lets the calling object decide where to attach
  // the schedule. In this case, the main program will use "nil" which is
  // the top of the tree.

  [super activateIn: swarmContext];
  [displaySchedule activateIn: self];

  // Here we return the swarm activity, i.e. what's ready to run.
  return [self getSwarmActivity];
}

@end








