// ObserverSwarm.m

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ObserverSwarm

+createBegin: (id) aZone {
  ObserverSwarm * obj;

  obj = [super createBegin: aZone];
  obj->displayFrequency = 4;
  obj->zoomFactor = 20;
  return obj;
}

-createEnd {
  return [super createEnd];
}

-buildObjects {

  [super buildObjects];
  modelSwarm = [ModelSwarm create: self];

  [controlPanel setStateStopped];
  [modelSwarm buildObjects];
  
  colorMap = [Colormap create: self];
  [colorMap setColor: 0 ToName: "red"];
  [colorMap setColor: 1 ToName: "blue"];
  [colorMap setColor: 2 ToName: "orange"];
  [colorMap setColor: 3 ToName: "black"];

  worldRaster = [ZoomRaster create: self];
  [worldRaster setColormap: colorMap];
  [worldRaster setZoomFactor: zoomFactor];
  [worldRaster setWidth: [modelSwarm getSize] Height: [modelSwarm getSize]];
  [worldRaster setWindowTitle: "Spatial PD"];
  [worldRaster pack];

  worldDisplay = [Object2dDisplay createBegin: self];
  [worldDisplay setDisplayWidget: worldRaster];
  [worldDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [worldDisplay setObjectCollection: [modelSwarm getPlayers]];
  [worldDisplay setDisplayMessage: M(drawSelfOn:)];
  worldDisplay = [worldDisplay createEnd];

  [worldRaster setButton: ButtonRight Client: worldDisplay Message: M(makeProbeAtX:Y:)];  
  numGraph = [EZGraph createBegin: self];
  [numGraph setTitle: "Number of all-C"];
  [numGraph setAxisLabelsX: "Time" Y: "Number"];
  numGraph = [numGraph createEnd];
  
  [numGraph createSequence: "all-C" withFeedFrom: modelSwarm andSelector: M(getNum0)]; 
  [numGraph createSequence: "TFT" withFeedFrom: modelSwarm andSelector: M(getNum1)];
  [numGraph createSequence: "aTFT" withFeedFrom: modelSwarm andSelector: M(getNum2)];
  [numGraph createSequence: "all-D" withFeedFrom: modelSwarm andSelector: M(getNum3)];

  [controlPanel setStateStopped];
  return self;
}

-buildActions {
  [super buildActions];

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: modelSwarm         message: M(step)];
  [displayActions createActionTo: worldDisplay       message: M(display)];
  [displayActions createActionTo: worldRaster        message: M(drawSelf)];
  [displayActions createActionTo: numGraph           message: M(step)];
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: 1]; // note frequency!
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
 
  return self;
}

-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  [displaySchedule activateIn: self];
  return [self getSwarmActivity];
}

@end








